import java.util.*;
import java.net.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;

//possible times are:
// 0 (red-wait), 1 (blue-register), 2 (green-can_alarm), 3 (yellow-can_alarm)

public class ControlMonitor extends Frame implements ActionListener{
	// hard coded, must match PeopleDB
	private String names[] = {"Shivashankar Balu", "Ding Liu",           "Ross Messing",
	                          "Matthew Post",      "Arrvindh Shriraman", "Jacob Balazar",
	                          "Piotr Faliszewski", "Michael Spear",      "Wenzhao Tan",
	                          "Anustup Choudhury", "Pin Lu",             "Tanushree Mittal",
	                          "Lior Privman",      "Benjamin VanDurme"};

	private Choice nameChoice;
	private Button loginButton;
	private Button tmpAlarmButton; //just for debug?
	public String currentLoginName;
	private String dbServer, domain;
	private TextArea display;
	private String timeServer;
	private TimeListenerClient timethread;
	public int time;
	
	public ControlMonitor(String d, String s, String t){
		super ("ControlMonitor..."+d.toUpperCase());
		dbServer =s;
		domain = d;
		currentLoginName="";
		
		
		setLayout(new GridLayout(3,1)); //3 rows, 1 column
		
		//create awt objects
		loginButton = new Button("Register");
		loginButton.addActionListener(this);
		
		tmpAlarmButton = new Button("Alarm!!!");
		tmpAlarmButton.addActionListener(this);

		nameChoice = new Choice();
		for (int i=0;i<names.length;i++)
			nameChoice.add(names[i]);
		
		display = new TextArea("", 0, 0, TextArea.SCROLLBARS_VERTICAL_ONLY);
		
		//add awt objects
		add(nameChoice);
		add(loginButton);
		add(tmpAlarmButton);
		add(display);
		
		setSize(300, 150);
		setVisible(true);
		
		//set up the time server connection
		timeServer=t;
		time=0;
		timethread = new TimeListenerClient(display, timeServer, this);
		timethread.start();
	}
	
	//button click actions
	public void actionPerformed(ActionEvent e){
		Socket client;
		DataOutputStream output;
		String message="";
		
		if (e.getSource()==loginButton){
			if (time==1){//only allow login during blue
				if (currentLoginName.equals("")){
					currentLoginName = names[nameChoice.getSelectedIndex()];
					message = "entered:STATUS_" + domain.toUpperCase() + ":"+currentLoginName;
				}
			}
		}
		if (e.getSource()==tmpAlarmButton){
			if (time==2||time==3){//alarm only in green or yellow
				if (!currentLoginName.equals(""))
					message = "arrested:" +currentLoginName;
			}
		}
		//send update to database
		if (!message.equals("")){
			try{
				client = new Socket(dbServer, 9999);
				output = new DataOutputStream(client.getOutputStream());
				output.writeUTF(message);
				client.close();
			}
			catch (IOException exc){
				exc.printStackTrace();
			}
		}
	}
	
	public static void main (String args[]){
		if (args.length!=3){
			System.out.println("ERROR: you must specify the name of the database server, the time server, and this machine's team\n");
			System.out.println("Proper usage:  java ControlMonitor green database.cs.rochester.edu time.cs.rochester.edu");
			System.out.println("Aborting");
			System.exit(1);
		}
		ControlMonitor cm = new ControlMonitor(args[0], args[1], args[2]);
		cm.addWindowListener(new CloseWindow());
	}
}

//thread to listen for click ticks
class TimeListenerClient extends Thread{
	String timeServer;
	TextArea displayarea;
	ControlMonitor parent;
	
	public TimeListenerClient(TextArea t, String s, ControlMonitor c){
		super();
		timeServer = s;
		displayarea=t;
		parent=c;
	}
	
	public void run(){
		String tick;
		try{
			//listen on 9990
	    ServerSocket servSock = new ServerSocket(9990);
	    //this registers the socket with the server
			Socket       sock= new Socket(timeServer,9991);
	    //registration done
			sock.close();
			
	    //main loop.. get a tick, parse it, send it to parent, and print it on
			// parent's screen
			while(true){
				sock = servSock.accept();
				tick = (new DataInputStream(sock.getInputStream())).readUTF();
				parent.time=new Integer(tick.split(":")[2]).intValue();
				if (parent.time==0)
					parent.currentLoginName="";
				displayarea.append(tick);
				sock.close();
	    }
		}
		catch( Exception e ){
	    e.printStackTrace();
		}
	}
}
